﻿#include <iostream>
#include <list>
#include <string>

using namespace std;

class invoice
{
private:
	int number_ = 0;
	double amount_;
	unsigned short month_ = 0;
	int category_ = 0;
public:
	static const char* categories[];
	static const char* months[];

	invoice(int number, double amount, unsigned short month, int category) : number_(number), amount_(amount),
	                                                                         month_(month), category_(category)
	{
	}

	int get_number() const
	{
		return number_;
	}

	unsigned short get_month() const
	{
		return month_;
	}

	int get_category() const
	{
		return category_;
	}

	void print() const
	{
		cout << "  Rechnung: " << number_ << endl;
		cout << "  Betrag: " << amount_ << "€" << endl;
		cout << "  Monat: " << months[month_] << endl;
		cout << "  Kategorie: " << categories[category_] << endl;
	}

	/**
 * \brief Implements the equality operator.
 */
	bool operator ==(const invoice& e) const
	{
		return number_ == e.number_;
	}

	/**
	 * \brief Implements the inequality operator.
	 */
	bool operator !=(const invoice& e) const { return !operator==(e); }
};

const char* invoice::categories[] = {"Tanken", "Nahrungsmittel", "Reinigungsmittel", "Urlaub", "Haustiere", "Bekleidung", "Wohnen", "Versicherung"};
const char* invoice::months[] = {"Januar", "Februar", "März", "April", "Mai", "Juni", "Juli", "August", "September", "Oktober", "November", "Dezember"};

template <typename T>
T ask_question(const string& question)
{
	cout << question << endl;
	cout << "> ";
	T value;
	cin >> value;
	return value;
}

list<invoice> invoices;

void create_invoice();

void output_invoices_order_by_month();

void delete_invoice();

bool compare_month_ascending(const invoice& first, const invoice& second)
{
	return first.get_month() < second.get_month();
}

int main()
{
	auto end_program = false;
	while (!end_program)
	{
		system("cls");
		cout << "Ausgabenverwaltung (" << invoices.size() << " Rechnungen)" << endl << endl;
		cout << "Bitte w\x84hlen Sie eine Option:" << endl;
		cout << "1: Erfassung einer Rechnung (inkl. Abfrage der Rechnungsdaten)" << endl;
		cout << "2: Alle Ausgaben, sortiert nach Monat, ausgeben" << endl;
		cout << "3: Summe der Ausgaben pro Kategorie ausgeben" << endl;
		cout << "4: Rechnung l\x94schen" << endl;
		cout << endl;
		cout << "0: Programm beenden" << endl;
		const auto input = ask_question<int>("Ihre Eingabe:");

		switch (input)
		{
		case 1:
			create_invoice();
			break;
		case 2:
			output_invoices_order_by_month();
			break;
		case 3:
			//TODO
			break;
		case 4:
			delete_invoice();
			break;
		case 0:
			end_program = true;
			break;
		default:
			cout << "Unerlaubte Eingabe" << endl;
			system("pause");
		}
	}
	return 0;
}

int get_month();
int get_category();

void create_invoice()
{
	system("cls");
	cout << "Eine neue Rechnung erstellen." << endl;
	cout << "------------------------------------"<<endl;
	const auto invoice_no = invoices.empty() ? 1 : invoices.back().get_number() + 1;
	const auto amount = ask_question<double>("Bitte geben Sie den Betrag in Euro ein.");
	const auto month = get_month();
	const auto category = get_category();
	const auto i = invoice(invoice_no, amount, month, category);
	invoices.push_back(i);
}


void output_invoices_order_by_month()
{
	system("cls");
	cout << "Alle Rechnungen nach Monat sortiert:"<<endl;
	cout << "------------------------------------"<<endl;
	auto temp = list<invoice>(invoices);
	temp.sort(compare_month_ascending);
	for (auto invoice : temp)
	{
		invoice.print();
		cout << "------------------------------------" << endl;
	}
	system("pause");
}

void delete_invoice()
{
	const auto invoice_number = ask_question<int>("Bitte geben Sie die Rechnungsnummer ein");
	for (auto invoice : invoices)
	{
		if (invoice.get_number() == invoice_number)
		{
			invoices.remove(invoice);
			return;
		}
	}
	cout << "Rechnung nicht gefunden." << endl;
	system("pause");
}

int get_month()
{
	while (true)
	{
		cout << "Bitte w\x84hlen sie einen Monat aus:" << endl;
		auto index = 0;
		for (auto month : invoice::months)
		{
			cout << ++index << ": " << month << endl;
		}
		const auto answer = ask_question<unsigned int>("");
		if (answer > 0 && answer <= 12)
		{
			return answer - 1;
		}
		cout << "Ung\x81ltige Eingabe" << endl << endl;
	}
}

int get_category()
{
	while (true)
	{
		cout << "Bitte w\x84hlen sie eine Kategorie aus:" << endl;
		auto index = 0;
		for (auto category : invoice::categories)
		{
			cout << ++index << ": " << category << endl;
		}
		const auto answer = ask_question<int>("");
		if (answer > 0 && answer <= 8)
		{
			return answer - 1;
		}
		cout << "Ung\x81ltige Eingabe" << endl << endl;
	}
}
